import logging
from functools import partial

from django.core.cache import cache
from django.http.response import sync_to_async
from terminusgps.wialon import flags
from terminusgps.wialon.session import WialonAPIError, WialonSession

from .forms import NotificationDispatchForm

logger = logging.getLogger(__name__)


@sync_to_async
def get_phone_numbers(
    form: NotificationDispatchForm, session: WialonSession, timeout: int = 300
) -> list[str]:
    """
    Returns a list of unit assigned phone numbers by id.

    :param form: A Wialon notification dispatch form.
    :type form: ~terminusgps_notifier.forms.WialonNotificationDispatchForm
    :param session: Active Wialon API session.
    :type session: ~terminusgps.wialon.session.WialonSession
    :param timeout: Cache timeout in seconds. Default is ``300`` (5min).
    :type timeout: int
    :returns: A list of phone numbers.
    :rtype: list[str]

    """
    unit_id = form.cleaned_data["unit_id"]
    driver_phones = cache.get_or_set(
        f"{unit_id}_get_driver_phone_numbers",
        partial(get_driver_phone_numbers, unit_id, session),
        timeout=timeout,
    )
    cfield_phones = cache.get_or_set(
        f"{unit_id}_get_cfield_phone_numbers",
        partial(get_cfield_phone_numbers, unit_id, session),
        timeout=timeout,
    )
    return list(frozenset(driver_phones + cfield_phones))


def get_driver_phone_numbers(
    unit_id: int, session: WialonSession
) -> list[str]:
    """
    Returns a list of the unit's attached driver phone numbers.

    :param unit_id: Wialon unit id.
    :type unit_id: int
    :param session: Active Wialon API session.
    :type session: ~terminusgps.wialon.session.WialonSession
    :returns: A list of phone numbers.
    :rtype: list[str]

    """
    try:
        logger.debug(
            f"Calling the Wialon API to retrieve driver phones for unit #{unit_id}..."
        )
        drivers = session.wialon_api.resource_get_unit_drivers(
            **{"unitId": unit_id}
        )
        return (
            [driver[0].get("ph") for driver in drivers.values()]
            if drivers.values()
            else []
        )
    except WialonAPIError as e:
        logger.warning(e)
        return []


def get_cfield_phone_numbers(
    unit_id: int, session: WialonSession, cfield_key: str = "to_number"
) -> list[str]:
    """
    Returns a list of the unit's attached driver phone numbers.

    :param unit_id: Wialon unit id.
    :type unit_id: int
    :param session: Active Wialon API session.
    :type session: ~terminusgps.wialon.session.WialonSession
    :param cfield_key: Custom field key containing a comma-separated list of phone numbers. Default is :py:obj:`"to_number"`.
    :type cfield_key: str
    :returns: A list of phone numbers.
    :rtype: list[str]

    """
    try:
        logger.debug(
            f"Calling the Wialon API to retrieve cfield phones for unit #{unit_id}..."
        )
        search = session.wialon_api.core_search_item(
            **{"id": unit_id, "flags": flags.DataFlag.UNIT_CUSTOM_FIELDS}
        )
        if item := search.get("item"):
            if cfields := item.get("flds"):
                for cfield in cfields.values():
                    if cfield["n"] == cfield_key:
                        return (
                            cfield["v"].split(",")
                            if "," in cfield["v"]
                            else [cfield["v"]]
                        )
        return []
    except WialonAPIError as e:
        logger.warning(e)
        return []



import decimal
import typing
from collections.abc import Sequence

from django.contrib.auth import get_user_model
from django.db import models
from django.db.models import F
from django.utils.translation import gettext_lazy as _
from terminusgps.wialon.session import WialonSession


class TerminusGPSNotifierCustomer(models.Model):
    messages_count = models.PositiveIntegerField(default=0)
    messages_limit = models.PositiveIntegerField(default=500)

    user = models.ForeignKey(
        get_user_model(),
        on_delete=models.CASCADE,
        related_name="notifier_customer",
    )
    tax_rate = models.DecimalField(
        decimal_places=4,
        default=decimal.Decimal("0.0825"),
        help_text=_("Enter a tax rate as a decimal."),
        max_digits=12,
    )
    sub_total = models.DecimalField(
        decimal_places=2,
        default=decimal.Decimal("60.00"),
        help_text=_("Enter a sub total as a decimal."),
        max_digits=12,
    )
    tax_total = models.GeneratedField(
        db_persist=True,
        expression=(F("sub_total") * (F("tax_rate") + 1)) - F("sub_total"),
        help_text=_("Automatically generated tax amount."),
        output_field=models.DecimalField(decimal_places=2, max_digits=12),
    )
    grand_total = models.GeneratedField(
        db_persist=True,
        expression=F("sub_total") * (F("tax_rate") + 1),
        help_text=_("Automatically generated grand total amount (sub+tax)."),
        output_field=models.DecimalField(decimal_places=2, max_digits=12),
    )
    subscription = models.ForeignKey(
        "terminusgps_payments.Subscription",
        blank=True,
        default=None,
        help_text=_("Select an Authorizenet subscription from the list."),
        null=True,
        on_delete=models.CASCADE,
        related_name="notifier_customer",
    )

    class Meta:
        verbose_name = _("customer")
        verbose_name_plural = _("customers")

    def __str__(self) -> str:
        return str(self.user)

    def get_resources_from_wialon(
        self, session: WialonSession, force: bool = False
    ) -> list[dict[str, typing.Any]]:
        params = {
            "spec": {
                "itemsType": "avl_resource",
                "propName": "sys_name",
                "propValueMask": "*",
                "sortType": "sys_name",
                "propType": "property",
            },
            "flags": 1,
            "force": int(force),
            "from": 0,
            "to": 0,
        }
        return session.wialon_api.core_search_items(**params).get("items", [])


class MessagePackage(models.Model):
    price = models.DecimalField(decimal_places=2, max_digits=12)
    count = models.IntegerField(default=0)
    limit = models.IntegerField(default=500)
    customer = models.ForeignKey(
        "terminusgps_notifier.TerminusGPSNotifierCustomer",
        on_delete=models.CASCADE,
        related_name="packages",
    )

    class Meta:
        verbose_name = _("message package")
        verbose_name_plural = _("message packages")

    def __str__(self) -> str:
        return f"MessagePackage #{self.pk}"


class WialonNotification(models.Model):
    class NotificationFlag(models.IntegerChoices):
        FIRST_MESSAGE = 0x0
        EVERY_MESSAGE = 0x1
        DISABLED = 0x2

    resource = models.ForeignKey(
        "terminusgps_notifier.WialonResource",
        on_delete=models.CASCADE,
        related_name="notifications",
    )

    wialon_id = models.IntegerField(blank=True)
    e = models.IntegerField(blank=True, default=1, verbose_name=_("enabled"))
    n = models.CharField(blank=True, max_length=50)
    txt = models.CharField(blank=True, max_length=1024)
    ta = models.IntegerField(blank=True, default=0)
    td = models.IntegerField(blank=True, default=0)
    ma = models.IntegerField(blank=True, default=0)
    mmtd = models.IntegerField(blank=True, default=0)
    cdt = models.IntegerField(blank=True, default=0)
    mast = models.IntegerField(blank=True, default=0)
    mpst = models.IntegerField(blank=True, default=0)
    cp = models.IntegerField(blank=True, default=0)
    fl = models.IntegerField(choices=NotificationFlag.choices, default=0x0)
    tz = models.IntegerField(blank=True, default=0)
    la = models.CharField(blank=True, default="en", max_length=2)
    ac = models.IntegerField(blank=True, default=0)
    un = models.JSONField(blank=True, default=list)
    d = models.CharField(blank=True)
    sch = models.JSONField(blank=True, default=dict)
    ctrl_sch = models.JSONField(blank=True, default=dict)
    trg = models.JSONField(blank=True, default=dict)
    act = models.JSONField(blank=True, default=list)
    ct = models.IntegerField(blank=True, default=0)
    mt = models.IntegerField(blank=True, default=0)

    class Meta:
        verbose_name = _("wialon notification")
        verbose_name_plural = _("wialon notifications")

    def __str__(self) -> str:
        return f"{self.n or self.wialon_id}"

    def save(self, **kwargs) -> None:
        if session := kwargs.pop("session", None):
            if self.wialon_id is not None:
                data = self.pull(session)[0]
                self.sync(data)
        return super().save(**kwargs)

    def pull(self, session: WialonSession) -> list[dict[str, typing.Any]]:
        return session.wialon_api.resource_get_notification_data(
            **{"itemId": self.resource.pk, "col": [self.wialon_id]}
        )

    def sync(self, data: dict[str, typing.Any]) -> None:
        self.n = data["n"]
        self.txt = data["txt"]
        self.ta = data["ta"]
        self.td = data["td"]
        self.ma = data["ma"]
        self.mmtd = data["mmtd"]
        self.cdt = data["cdt"]
        self.mast = data["mast"]
        self.mpst = data["mpst"]
        self.cp = data["cp"]
        self.fl = data["fl"]
        self.tz = data["tz"]
        self.la = data["la"]
        self.ac = data["ac"]
        self.d = data["d"]
        self.sch = data["sch"]
        self.ctrl_sch = data["ctrl_sch"]
        self.un = data["un"]
        self.act = data["act"]
        self.trg = data["trg"]
        self.ct = data["ct"]
        self.mt = data["mt"]


class WialonObject(models.Model):
    id = models.PositiveBigIntegerField(primary_key=True)
    nm = models.CharField(blank=True, max_length=50)
    mu = models.IntegerField(blank=True, default=0)
    uacl = models.IntegerField(blank=True, default=0)

    class Meta:
        abstract = True

    def __str__(self) -> str:
        return f"{self.nm or self.id}"

    def save(self, **kwargs) -> None:
        if session := kwargs.pop("session", None):
            if self.pk:
                data = self.pull(session)
                self.sync(data)
        return super().save(**kwargs)

    def pull(
        self, session: WialonSession, flags: int = 1
    ) -> dict[str, typing.Any]:
        return session.wialon_api.core_search_item(
            **{"id": self.pk, "flags": flags}
        )

    def sync(self, data: dict[str, typing.Any]) -> None:
        self.nm = data["item"]["nm"]
        self.mu = data["item"]["mu"]
        self.uacl = data["item"]["uacl"]


class WialonResource(WialonObject):
    class Meta:
        verbose_name = _("wialon resource")
        verbose_name_plural = _("wialon resources")

    def get_notification_data(
        self, session: WialonSession, ids: Sequence[str] | None = None
    ) -> list[dict[str, typing.Any]]:
        params = {"itemId": self.pk}
        if ids is not None:
            params["col"] = ids
        return session.wialon_api.resource_get_notification_data(**params)


class WialonUnit(WialonObject):
    class Meta:
        verbose_name = _("wialon unit")
        verbose_name_plural = _("wialon units")


class WialonUnitGroup(WialonObject):
    class Meta:
        verbose_name = _("wialon unit group")
        verbose_name_plural = _("wialon unit groups")



import asyncio
import logging
import warnings

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured, ValidationError
from django.http import Http404, HttpRequest, HttpResponse
from django.utils.decorators import method_decorator
from django.utils.module_loading import import_string
from django.views.decorators.csrf import csrf_exempt
from django.views.generic import View
from terminusgps.wialon.session import WialonSession

from terminusgps_notifier.dispatchers import NotificationDispatcher
from terminusgps_notifier.forms import NotificationDispatchForm
from terminusgps_notifier.validators import validate_e164_phone_number
from terminusgps_notifier.wialon import get_phone_numbers

logger = logging.getLogger(__name__)


class HealthCheckView(View):
    content_type = "text/plain"
    http_method_names = ["get"]

    async def get(self, request: HttpRequest, *args, **kwargs) -> HttpResponse:
        """Responds with status code 200."""
        return HttpResponse("I'm alive\n".encode("utf-8"), status=200)


@method_decorator(csrf_exempt, name="dispatch")
class NotificationDispatchView(View):
    content_type = "text/plain"
    http_method_names = ["get", "post"]

    @staticmethod
    async def clean_phones(phones: list[str]) -> list[str]:
        cleaned = []
        for phone in phones:
            try:
                validate_e164_phone_number(phone)
                cleaned.append(phone)
            except ValidationError:
                logger.warning(f"Improperly formatted phone number: {phone}")
                continue
        return cleaned

    async def get_phones(self, form: NotificationDispatchForm) -> list[str]:
        with WialonSession(token=settings.WIALON_TOKEN) as session:
            dirty_phones = await get_phone_numbers(form, session)
            return await self.clean_phones(dirty_phones)

    async def get_dispatchers(
        self, form: NotificationDispatchForm
    ) -> list[NotificationDispatcher]:
        return [dispatcher(form) for dispatcher in self.dispatcher_classes]

    async def send_notifications(
        self, phones: list[str], dispatchers: list[NotificationDispatcher]
    ) -> HttpResponse:
        for dispatcher in dispatchers:
            try:
                tasks = [
                    dispatcher.send_notification(
                        to_number=phone, method=self.kwargs["method"]
                    )
                    for phone in phones
                ]
                await asyncio.gather(*tasks)
                logger.info(f"Dispatched via {type(dispatcher).__name__}")
                return HttpResponse(status=200)
            except Exception as error:
                logger.warning(
                    f"{type(dispatcher).__name__} failed: '{error}'"
                )
        logger.error(
            f"All dispatchers failed for method: '{self.kwargs['method']}'"
        )
        return HttpResponse(status=500)

    async def dispatch(
        self, request: HttpRequest, *args, **kwargs
    ) -> HttpResponse:
        try:
            method = kwargs.get("method")
            if method not in settings.NOTIFICATION_DISPATCHERS:
                raise ValueError(f"Invalid method: '{method}'")
            self.dispatcher_classes = []
            for dispatcher_path in settings.NOTIFICATION_DISPATCHERS[method]:
                dispatcher_cls = import_string(dispatcher_path)
                self.dispatcher_classes.append(dispatcher_cls)
            return await super().dispatch(request, *args, **kwargs)
        except ImportError as e:
            raise ImproperlyConfigured(e)
        except ValueError as e:
            raise Http404(e)

    async def post(
        self, request: HttpRequest, *args, **kwargs
    ) -> HttpResponse:
        """
        Dispatches notifications to target phone numbers.

        Returns 200 if notifications were queued successfully.

        Returns 500 if all dispatchers failed to queue notifications.

        Returns 4XX in any other case.

        """
        form = NotificationDispatchForm(request.POST)
        if not form.is_valid():
            return HttpResponse(status=406)
        phones = await self.get_phones(form)
        if not phones:
            return HttpResponse(status=200)
        dispatchers = await self.get_dispatchers(form)
        return await self.send_notifications(phones, dispatchers)

    async def get(self, request: HttpRequest, *args, **kwargs) -> HttpResponse:
        """Deprecated. Use POST instead."""
        warnings.warn(
            f"{type(self).__name__}.get() will return 405 in the future, use {type(self).__name__}.post() instead",
            category=FutureWarning,
            stacklevel=2,
        )
        form = NotificationDispatchForm(request.GET)
        if not form.is_valid():
            return HttpResponse(status=406)
        phones = await self.get_phones(form)
        if not phones:
            return HttpResponse(status=200)
        dispatchers = await self.get_dispatchers(form)
        return await self.send_notifications(phones, dispatchers)



from django.core.exceptions import ValidationError
from django.utils.translation import gettext_lazy as _

__all__ = ["validate_e164_phone_number"]


def validate_e164_phone_number(value: str) -> None:
    """Raises :py:exec:`~django.core.exceptions.ValidationError` if the value wasn't a properly formatted E.164 phone number."""
    if not value.startswith("+"):
        raise ValidationError(
            _("E.164 phone number must start with a '+', got '%(char)s'"),
            code="invalid",
            params={"char": value[0]},
        )
    if not value.removeprefix("+").isdigit():
        raise ValidationError(
            _(
                "E.164 phone number must be entirely digits following '+', got '%(value)s'"
            ),
            code="invalid",
            params={"value": value.removeprefix("+")},
        )
    if len(value.removeprefix("+")) > 15:
        raise ValidationError(
            _(
                "E.164 phone number cannot be greater than 15 characters in length, got %(len)s."
            ),
            code="invalid",
            params={"len": len(value.removeprefix("+"))},
        )



import base64
import os
import sys
from pathlib import Path
from socket import gethostbyname, gethostname

from authorizenet.constants import constants

BASE_DIR = Path(__file__).resolve().parent.parent

ALLOWED_HOSTS = [
    ".terminusgps.com",
    ".elb.amazonaws.com",
    gethostbyname(gethostname()),
]
ASGI_APPLICATION = "src.asgi.application"
AWS_PINPOINT_CONFIGURATION_ARN = os.getenv("AWS_PINPOINT_CONFIGURATION_ARN")
AWS_PINPOINT_MAX_PRICE_SMS = os.getenv("AWS_PINPOINT_MAX_PRICE_SMS")
AWS_PINPOINT_MAX_PRICE_VOICE = os.getenv("AWS_PINPOINT_MAX_PRICE_VOICE")
AWS_PINPOINT_POOL_ARN = os.getenv("AWS_PINPOINT_POOL_ARN")
AWS_PINPOINT_PROTECT_ID = os.getenv("AWS_PINPOINT_PROTECT_ID")
TWILIO_ACCOUNT_SID = os.getenv("TWILIO_ACCOUNT_SID")
TWILIO_AUTH_TOKEN = os.getenv("TWILIO_AUTH_TOKEN")
TWILIO_FROM_NUMBER = os.getenv("TWILIO_FROM_NUMBER")
CSRF_COOKIE_SECURE = True
DEBUG = False
DEFAULT_AUTO_FIELD = "django.db.models.BigAutoField"
LANGUAGE_CODE = "en-us"
MERCHANT_AUTH_ENVIRONMENT = constants.PRODUCTION
MERCHANT_AUTH_LOGIN_ID = os.getenv("MERCHANT_AUTH_LOGIN_ID")
MERCHANT_AUTH_TRANSACTION_KEY = os.getenv("MERCHANT_AUTH_TRANSACTION_KEY")
MERCHANT_AUTH_VALIDATION_MODE = "liveMode"
DJANGO_ENCRYPTED_FIELD_ALGORITHM = os.getenv(
    "DJANGO_ENCRYPTED_FIELD_ALGORITHM", "SS20"
)
DJANGO_ENCRYPTED_FIELD_KEY = base64.b64decode(
    os.getenv("DJANGO_ENCRYPTED_FIELD_KEY", "")
)
ROOT_URLCONF = "src.urls"
SECRET_KEY = os.getenv("SECRET_KEY")
SECURE_PROXY_SSL_HEADER = ("HTTP_X_FORWARDED_PROTO", "https")
SECURE_SSL_REDIRECT = False
SESSION_COOKIE_SECURE = True
STATIC_URL = "static/"
TIME_ZONE = "US/Central"
USE_I18N = True
USE_TZ = True
USE_X_FORWARDED_HOST = True
WIALON_TOKEN = os.getenv("WIALON_TOKEN")
WSGI_APPLICATION = "src.wsgi.application"
NOTIFICATION_DISPATCHERS = {
    "sms": ["terminusgps_notifier.dispatchers.AWSNotificationDispatcher"],
    "voice": [
        "terminusgps_notifier.dispatchers.AWSNotificationDispatcher",
        "terminusgps_notifier.dispatchers.TwilioNotificationDispatcher",
    ],
}

LOGGING = {
    "version": 1,
    "disable_existing_loggers": "test" in sys.argv,
    "formatters": {
        "verbose": {
            "format": "%(asctime)s [%(process)d] [%(module)s] [%(levelname)s] %(message)s",
            "datefmt": "[%Y-%m-%d %H:%M:%S%z]",
            "class": "logging.Formatter",
        },
        "simple": {
            "format": "%(asctime)s [%(levelname)s] %(message)s",
            "datefmt": "[%Y-%m-%d %H:%M:%S%z]",
            "class": "logging.Formatter",
        },
    },
    "handlers": {
        "console": {"class": "logging.StreamHandler", "formatter": "simple"},
        "console_verbose": {
            "class": "logging.StreamHandler",
            "formatter": "verbose",
        },
    },
    "root": {"handlers": ["console"], "level": "WARNING"},
    "loggers": {
        "django": {
            "handlers": ["console"],
            "level": os.getenv("DJANGO_LOG_LEVEL", "INFO"),
            "propagate": False,
        },
        "terminusgps_notifier": {
            "handlers": ["console_verbose"],
            "level": os.getenv("NOTIFIER_LOG_LEVEL", "DEBUG"),
            "propagate": False,
        },
    },
}


DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.postgresql",
        "NAME": os.getenv("DB_NAME"),
        "HOST": os.getenv("DB_HOST"),
        "USER": os.getenv("DB_USER"),
        "PASSWORD": os.getenv("DB_PASSWORD"),
        "PORT": os.getenv("DB_PORT", 5432),
        "OPTIONS": {"client_encoding": "UTF8"},
        "CONN_MAX_AGE": None,
    }
}

CACHES = {
    "default": {
        "BACKEND": "django.core.cache.backends.redis.RedisCache",
        "LOCATION": "redis://127.0.0.1:6379",
        "TIMEOUT": 60 * 5,
    }
}

INSTALLED_APPS = [
    "django.contrib.admin",
    "django.contrib.admindocs",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.messages",
    "django.contrib.sessions",
    "django.contrib.staticfiles",
    "django.forms",
    "terminusgps_notifier.apps.TerminusgpsNotifierConfig",
]

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
            ],
            "autoescape": False,
        },
    }
]

MIDDLEWARE = [
    "django.middleware.security.SecurityMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
]



# Generated by Django 6.0.2 on 2026-03-03 19:25

from decimal import Decimal
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('terminusgps_notifier', '0002_alter_wialonnotification_act'),
    ]

    operations = [
        migrations.AlterField(
            model_name='terminusgpsnotifiercustomer',
            name='tax_rate',
            field=models.DecimalField(decimal_places=4, default=Decimal('0.0825'), help_text='Enter a tax rate as a decimal.', max_digits=12),
        ),
    ]



from django.urls import path

from . import views

urlpatterns = [
    path("health/", views.HealthCheckView.as_view(), name="health check"),
    path(
        "notify/<str:method>/",
        views.NotificationDispatchView.as_view(),
        name="notify",
    ),
]



from django import forms
from django.db import models
from django.utils.translation import gettext_lazy as _


class WialonNotificationTrigger(models.TextChoices):
    GEOFENCE = "geozone", _("Geofence")
    ADDRESS = "address", _("Address")
    SPEED = "speed", _("Speed")
    ALARM = "alarm", _("Alarm")
    DIGITAL_INPUT = "digital_input", _("Digital input")
    PARAMETER = "msg_param", _("Parameter in a message")
    SENSOR = "sensor_value", _("Sensor value")
    OUTAGE = "outage", _("Connection loss")
    SMS = "sms", _("SMS")
    INTERPOSITION = "interposition", _("Interposition")
    EXCESS = "msgs_counter", _("Excess of messages")
    ROUTE = "route_control", _("Route progress")
    DRIVER = "driver", _("Driver")
    TRAILER = "trailer", _("Trailer")
    MAINTENANCE = "service_intervals", _("Maintenance")
    FUEL_FILL = "fuel_filling", _("Fuel filling")
    FUEL_DRAIN = "fuel_theft", _("Fuel draining")
    HEALTH = "health_check", _("Health check")
    COMBO = "expression", _("Combination")


class WialonNotificationAction(models.TextChoices):
    EMAIL = "email", _("Email")
    SMS = "sms", _("SMS")
    MESSAGE = "message", _("Message")
    MOBILE = "mobile_apps", _("Send mobile notification")
    REQUEST = "push_messages", _("Send a request")
    TELEGRAM = "messenger_messages", _("Send notification to Telegram")
    EVENT = "event", _("Register event for unit")
    COMMAND = "exec_cmd", _("Send a command")
    ACCESS = "user_access", _("Change access to units")
    SET_COUNTER = "counter", _("Set counter value")
    STORE_COUNTER = "store_counter", _("Store counter value as a parameter")
    STATUS = "status", _("Register unit status")
    GROUPS = "group_manipulation", _("Add or remove units from groups")
    REPORT = "email_report", _("Send a report by email")
    RIDE = "route_control", _("Create a ride")
    SEPARATE_DRIVER = "drivers_reset", _("Separate driver")
    SEPARATE_TRAILER = "trailers_reset", _("Separate trailer")
    TASK = "create_task", _("Create task")


class NotificationDispatchForm(forms.Form):
    """
    A form for dispatching Wialon unit notifications.

    Attributes:
        Required:
            - unit_id: An 8-digit Wialon unit id for phone number lookup.
            - user_id: A user id for Wialon API token retrieval.
            - message: A 1024-character max message to dispatch.
            - msg_time_int: Notification trigger time as a UNIX-timestamp.

        Optional:
            - dry_run: Whether to dispatch the notification as a dry run.
            - unit_name: Name of the unit that triggered the notification.
            - location: Location of the notification trigger.
            - date_format: Date format string.

    """

    user_id = forms.IntegerField()
    unit_id = forms.IntegerField()
    message = forms.CharField(max_length=1024)
    msg_time_int = forms.IntegerField()
    dry_run = forms.BooleanField(initial=False, required=False)
    unit_name = forms.CharField(required=False)
    location = forms.CharField(required=False)
    date_format = forms.CharField(required=False, initial="%Y-%m-%d %H:%M:%S")


class WialonNotificationScheduleForm(forms.Form):
    f1 = forms.IntegerField(min_value=0)
    f2 = forms.IntegerField(min_value=0)
    t1 = forms.IntegerField(min_value=0)
    t2 = forms.IntegerField(min_value=0)
    m = forms.IntegerField(min_value=0)
    y = forms.IntegerField(min_value=0)
    w = forms.IntegerField(min_value=0)


class WialonNotificationTriggerForm(forms.Form):
    t = forms.ChoiceField(choices=WialonNotificationTrigger.choices)
    p = forms.JSONField(required=False)

    def clean(self):
        cleaned_data = super().clean()
        if cleaned_data:
            if cleaned_data["p"] is None:
                cleaned_data["p"] = {}


class WialonNotificationActionForm(forms.Form):
    t = forms.ChoiceField(choices=WialonNotificationAction.choices)
    p = forms.JSONField(required=False)

    def clean(self):
        cleaned_data = super().clean()
        if cleaned_data:
            if cleaned_data["p"] is None:
                cleaned_data["p"] = {}



# Generated by Django 6.0.2 on 2026-02-20 18:32

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('terminusgps_notifier', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='wialonnotification',
            name='act',
            field=models.JSONField(blank=True, default=list),
        ),
    ]



import datetime
from abc import ABC, abstractmethod

import aioboto3
from django.conf import settings
from django.http.response import sync_to_async
from django.template.loader import render_to_string
from twilio.http.async_http_client import AsyncTwilioHttpClient
from twilio.rest import Client
from twilio.twiml.voice_response import VoiceResponse

from .forms import NotificationDispatchForm


class NotificationDispatcher(ABC):
    def __init__(self, form: NotificationDispatchForm) -> None:
        if not form.is_valid():
            raise ValueError("Form must be valid")
        self.form = form

    @sync_to_async
    def render_message(self, template_name: str) -> str:
        msg_time_int = self.form.cleaned_data["msg_time_int"]
        date = datetime.datetime.fromtimestamp(float(msg_time_int))
        context = self.form.cleaned_data.copy()
        context.update({"date": date})
        return render_to_string(template_name, context)

    @abstractmethod
    async def send_voice(
        self, to_number: str, dry_run: bool = False, **kwargs
    ) -> str | None:
        raise NotImplementedError("Subclasses must implement this method.")

    @abstractmethod
    async def send_sms(
        self, to_number: str, dry_run: bool = False, **kwargs
    ) -> str | None:
        raise NotImplementedError("Subclasses must implement this method.")

    async def send_notification(
        self, to_number: str, method: str
    ) -> str | None:
        dry_run = self.form.cleaned_data.get("dry_run", False)
        match method:
            case "sms":
                return await self.send_sms(to_number, dry_run)
            case "voice":
                return await self.send_voice(to_number, dry_run)
            case _:
                raise ValueError(f"Invalid method: '{method}'")


class AWSNotificationDispatcher(NotificationDispatcher):
    def __init__(
        self, form: NotificationDispatchForm, region_name: str = "us-east-1"
    ) -> None:
        super().__init__(form=form)
        self.session = aioboto3.Session()
        self.service = "pinpoint-sms-voice-v2"
        self.region_name = region_name

    async def send_voice(
        self,
        to_number: str,
        dry_run: bool = False,
        *,
        message_type: str = "TEXT",
        voice_id: str = "MATTHEW",
        pool_arn: str | None = None,
        config_arn: str | None = None,
        mppm: str | None = None,
        protect_id: str | None = None,
    ):
        message = await self.render_message(
            "terminusgps_notifier/message_voice.txt"
        )
        async with self.session.client(
            self.service, region_name=self.region_name
        ) as client:
            return await client.send_voice_message(
                **{
                    "DestinationPhoneNumber": to_number,
                    "OriginationIdentity": pool_arn
                    or settings.AWS_PINPOINT_POOL_ARN,
                    "MessageBody": message,
                    "MessageBodyTextType": message_type,
                    "VoiceId": voice_id,
                    "ConfigurationSetName": config_arn
                    or settings.AWS_PINPOINT_CONFIGURATION_ARN,
                    "MaxPricePerMinute": mppm
                    or settings.AWS_PINPOINT_MAX_PRICE_VOICE,
                    "DryRun": dry_run,
                    "ProtectConfigurationId": protect_id
                    or settings.AWS_PINPOINT_PROTECT_ID,
                }
            )

    async def send_sms(
        self,
        to_number: str,
        dry_run: bool = False,
        *,
        ttl: int = 300,
        pool_arn: str | None = None,
        config_arn: str | None = None,
        mpps: str | None = None,
        protect_id: str | None = None,
    ):
        message = await self.render_message(
            "terminusgps_notifier/message_sms.txt"
        )
        async with self.session.client(
            self.service, region_name=self.region_name
        ) as client:
            return await client.send_text_message(
                **{
                    "DestinationPhoneNumber": to_number,
                    "OriginationIdentity": pool_arn
                    or settings.AWS_PINPOINT_POOL_ARN,
                    "MessageBody": message,
                    "MessageType": "TRANSACTIONAL",
                    "ConfigurationSetName": config_arn
                    or settings.AWS_PINPOINT_CONFIGURATION_ARN,
                    "MaxPrice": mpps or settings.AWS_PINPOINT_MAX_PRICE_SMS,
                    "TimeToLive": ttl,
                    "DryRun": dry_run,
                    "ProtectConfigurationId": protect_id
                    or settings.AWS_PINPOINT_PROTECT_ID,
                }
            )


class TwilioNotificationDispatcher(NotificationDispatcher):
    async def send_voice(
        self,
        to_number: str,
        dry_run: bool = False,
        *,
        voice: str = "woman",
        from_number: str | None = None,
    ):
        if dry_run:
            return
        raw = await self.render_message(
            "terminusgps_notifier/message_voice.txt"
        )
        message = VoiceResponse()
        message.say(raw, voice=voice)
        client = Client(http_client=AsyncTwilioHttpClient())
        return await client.calls.create_async(
            to=to_number,
            from_=from_number or settings.TWILIO_FROM_NUMBER,
            twiml=message,
        )

    async def send_sms(
        self,
        to_number: str,
        dry_run: bool = False,
        *,
        from_number: str | None = None,
    ):
        if dry_run:
            return
        message = await self.render_message(
            "terminusgps_notifier/message_sms.txt"
        )
        client = Client(http_client=AsyncTwilioHttpClient())
        return await client.messages.create_async(
            to=to_number,
            from_=from_number or settings.TWILIO_FROM_NUMBER,
            body=message,
        )
